"""
    Archetypes items used in DataGridField unit testing and examples

"""
# Zope imports
from AccessControl import ClassSecurityInfo

# Plone imports
from Products.Archetypes import atapi

# Local imports
from Products.DataGridField import DGFMessageFactory as _
from Products.DataGridField import DataGridField, DataGridWidget
from Products.DataGridField.Column import Column
from Products.DataGridField.SelectColumn import SelectColumn
from Products.DataGridField.RadioColumn import RadioColumn
from Products.DataGridField.CheckboxColumn import CheckboxColumn
from Products.DataGridField.FixedColumn import FixedColumn
from Products.DataGridField.DataGridField import FixedRow
from Products.DataGridField.HelpColumn import HelpColumn
from Products.DataGridField.LinkColumn import LinkColumn
from Products.DataGridField.DateColumn import DateColumn
from Products.DataGridField.DatetimeColumn import DatetimeColumn
from Products.DataGridField.DatetimeLocalColumn import DatetimeLocalColumn
from Products.DataGridField.FileColumn import FileColumn
from Products.DataGridField.EmailColumn import EmailColumn
from Products.DataGridField.ColorColumn import ColorColumn
from Products.DataGridField.PasswordColumn import PasswordColumn
from Products.DataGridField.RangeColumn import RangeColumn
from Products.DataGridField.MonthColumn import MonthColumn
from Products.DataGridField.SearchColumn import SearchColumn
from Products.DataGridField.TimeColumn import TimeColumn
from Products.DataGridField.UrlColumn import UrlColumn
from Products.DataGridField.WeekColumn import WeekColumn
from Products.DataGridField.config import PKG_NAME


class DataGridDemoType(atapi.BaseContent):
    """ Very simple DataGridField demo.

    This class is used in unit testing, mainly to check old interface
    compatibility (without widget column definitions). Please see the
    more complex examples below.
    """
    security = ClassSecurityInfo()

    schema = atapi.BaseSchema + atapi.Schema((

        DataGridField('DemoField',
                searchable=True, # One unit tests checks whether text search works
                required=True,
                widget = DataGridWidget(),
                columns=('column1','column2','The third'),
                default=[
                {'column1':'a', 'column2':'b', 'The third':'c'},
                {'column1':'d', 'column2':'e', 'The third':'f'}
                ]
            ),
        ),
    )

    meta_type = portal_type = archetype_name = 'DataGridDemoType'

atapi.registerType(DataGridDemoType, PKG_NAME)


class DataGridDemoType2(atapi.BaseContent):
    """ Demo for different DataGridWidget columns

    This class is used in unit testing

    Check manual that:
        - Rows are inserted automatically when a value is filled in
        - Select column has sample 2 as a default value
    """
    security = ClassSecurityInfo()

    schema = atapi.BaseSchema + atapi.Schema((

        DataGridField('AutoInsertDemoField',
                searchable=True, # One unit tests checks whether text search works
                columns=("column1", "column2", "column3"),
                allow_empty_rows = False, # Must be false to make auto insert feature perform correctly
                widget = DataGridWidget(
                    auto_insert = True,
                    description="Automatically insert new rows when the last row is being filled. When you edit the last row, a new row is created automatically.",
                    description_msgid='DataGridDemoType_help_AutoInsertRows',
                    i18n_domain='datagridfield',
                    columns={
                    },
                 ),
         ),

        DataGridField('DemoField2',
                searchable=True, # One unit tests checks whether text search works
                columns=("column1", "column2", "select_sample"),
                widget = DataGridWidget(
                    description="Set default values for created rows. Choose SelectColumn value from the default dictionary",
                    description_msgid='DataGridDemoType_help_DemoField2',
                    i18n_domain='datagridfield',
                    columns={
                        'column1' : Column(_(u"Toholampi city rox"), required=True, col_description=_(u"Go Toholampi or go home.")),
                        'column2' : Column(_(u"My friendly name"), default=_(u"Some default text")),
                        'select_sample' : SelectColumn(_(u"Friendly name"), vocabulary="getSampleVocabulary", default="sample2")
                    },
                 ),
         ),

        DataGridField('DemoField3',
                columns=("column1", "column2"),
                widget = DataGridWidget(
                    description="Test radio and checkbox columns",
                    description_msgid='DataGridDemoType_help_DemoField3',
                    i18n_domain='datagridfield',
                    columns={
                        'column1' : RadioColumn("Radio column", vocabulary="getSampleVocabulary"),
                        'column2' : CheckboxColumn("Checkbox column", default='1')
                    },
                 ),
         ),

        DataGridField('DemoField4',
                columns=("text_column", "help_column"),
                widget = DataGridWidget(
                    description="Help column test",
                    description_msgid='DataGridDemoType_help_DemoField4',
                    i18n_domain='datagridfield',
                    columns={
                        'text_column' : Column("Text column"),
                        # Help is help.pt
                        'help_column' : HelpColumn("Help", _(u"See help here"), "help", "info.png")
                    },
                 ),
         ),

        DataGridField('DemoField5',
                columns=("link_column",),
                widget = DataGridWidget(
                    description="Set of links",
                    columns={
                        'link_column' : LinkColumn("Link column"),
                    },
                 ),
         ),

        DataGridField('DemoField6',
                columns=("date_column",),
                widget = DataGridWidget(
                    description="date column",
                    columns={
                        'date_column' : DateColumn("Date column"),
                    },
                 ),
         ),

        DataGridField('DemoField7',
                columns=("datetime_column",),
                widget = DataGridWidget(
                    description="datetime column",
                    columns={
                        'datetime_column' : DatetimeColumn("Datetime column"),
                    },
                 ),
         ),

        DataGridField('DemoField8',
                columns=("datetimelocal_column",),
                widget = DataGridWidget(
                    description="datetime column, local time zone",
                    columns={
                        'datetimelocal_column' : DatetimeLocalColumn("DatetimeLocal column"),
                    },
                 ),
         ),

        DataGridField('DemoField9',
                columns=("file_column",),
                widget = DataGridWidget(
                    description="file column",
                    columns={
                        'file_column' : FileColumn("File column"),
                    },
                 ),
         ),

        DataGridField('DemoField10',
                columns=("email_column",),
                widget = DataGridWidget(
                    description="email column",
                    columns={
                        'email_column' : EmailColumn("Email column"),
                    },
                 ),
         ),

        DataGridField('DemoField11',
                columns=("color_column",),
                widget = DataGridWidget(
                    description="color column",
                    columns={
                        'color_column' : ColorColumn("Color column"),
                    },
                 ),
         ),

        DataGridField('DemoField12',
                columns=("password_column",),
                widget = DataGridWidget(
                    description="password column",
                    columns={
                        'password_column' : PasswordColumn("Password column"),
                    },
                 ),
         ),

        DataGridField('DemoField13',
                columns=("range_column",),
                widget = DataGridWidget(
                    description="range column",
                    columns={
                        'range_column' : RangeColumn("Range column"),
                    },
                 ),
         ),

        DataGridField('DemoField14',
                columns=("month_column",),
                widget = DataGridWidget(
                    description="month column",
                    columns={
                        'month_column' : MonthColumn("Month column"),
                    },
                 ),
         ),

        DataGridField('DemoField15',
                columns=("search_column",),
                widget = DataGridWidget(
                    description="search column",
                    columns={
                        'search_column' : SearchColumn("Search column"),
                    },
                 ),
         ),

        DataGridField('DemoField16',
                columns=("time_column",),
                widget = DataGridWidget(
                    description="time column",
                    columns={
                        'time_column' : TimeColumn("Time column"),
                    },
                 ),
         ),

        DataGridField('DemoField17',
                columns=("url_column",),
                widget = DataGridWidget(
                    description="url column",
                    columns={
                        'url_column' : UrlColumn("Url column"),
                    },
                 ),
         ),

        DataGridField('DemoField18',
                columns=("week_column",),
                widget = DataGridWidget(
                    description="week column",
                    columns={
                        'week_column' : WeekColumn("Week column"),
                    },
                 ),
         ),


        ))

    meta_type = portal_type = archetype_name = 'DataGridDemoType2'

    def getSampleVocabulary(self):
        """Get a sample vocabulary
        """
        return atapi.DisplayList(

            (("sample", _(u"Sample value 1"),),
            ("sample2", _(u"Sample value 2"),),))

atapi.registerType(DataGridDemoType2, PKG_NAME)


class InvalidDataGridDemoType(atapi.BaseContent):
    """ DataGridField declaration with errors

    Errors should be detected run-time, with helpful error messages.

    This class is missing column definition select_sample in DataGridWidget
    """
    security = ClassSecurityInfo()

    schema = atapi.BaseSchema + atapi.Schema((
        DataGridField('DemoField',
                searchable = True,
                columns=("column1", "column2", "select_sample"),
                widget = DataGridWidget(
                    columns={
                        'column1' : Column(_(u"Toholampi city rox")),
                        'column2' : Column(_(u"My friendly name")),
                    },
          ),),

        ))

    meta_type = portal_type = archetype_name = 'InvalidDataGridDemoType'

atapi.registerType(InvalidDataGridDemoType, PKG_NAME)


class FixedRowsDemoType(atapi.BaseContent):
    """ Demostrate fixed rows usage

    This class is used in unit testing
    """
    security = ClassSecurityInfo()

    schema = atapi.BaseSchema + atapi.Schema((

        DataGridField('DemoField',
                widget = DataGridWidget(),
                columns=('column1','column2','The third'),
                fixed_rows = [
                    FixedRow(keyColumn="column1", initialData = { "column1" : "must-exist-1", "column2" : "bbb" }),
                    FixedRow(keyColumn="column2", initialData = { "column1" : "ddd", "column2" : "must-exist-2" }),
                ]
            ),

        DataGridField('RestrictedField',
                widget = DataGridWidget(),
                columns=('column1','column2','The third'),
                allow_insert=False,
                allow_delete=False,
                allow_reorder=False,
            ),

        DataGridField('predefinedSkills',
            searchable=True,
            columns=('skill', 'level'),
            fixed_rows = "getPredefinedSkillsData",
            allow_delete = False,
            allow_insert = False,
            allow_reorder = False,
            widget = DataGridWidget(
                label="Skills",
                label_msgid='DataGridDemoType_label_Skills',
                description="Language/technology/tool/method for which employer has special interest",
                description_msgid='DataGridDemoType_help_Skills',
                i18n_domain='datagridfield',
                columns= {
                    "skill" : FixedColumn(_(u"Skill")),
                    "level" : RadioColumn(_(u"Level"), vocabulary="getSkillLevels")
                }
            ),
            ),
        ))

    meta_type = portal_type = archetype_name = 'FixedRowsDemoType'

    def getSkillLevels(self):
        return atapi.DisplayList(
            (("bad", _(u"Bad"),),
             ("good", _(u"Good"),),
             ))

    def getPredefinedSkillsData(self):
        """ Generate fixed row key information """
        skills = [ "Python", "Perl", "XML", "Java", "Plone" ]
        rows = []
        for skill in skills:
            rows.append(FixedRow(keyColumn="skill", initialData={"skill" : skill, "level" : "bad"}))

        return rows


atapi.registerType(FixedRowsDemoType, PKG_NAME)







"""

    DataGridField initialzer.

    Copyright 2006-2007 DataGridField authors.

    Load all submodules and perform Zope security initialize for them.

"""

__author__ = "Mikko Ohtamaa <mikko@redinnovation.com>"
__docformat__ = 'epytext'


# Zope imports
from zope.i18nmessageid import MessageFactory

# Plone imports
from Products.CMFCore.DirectoryView import registerDirectory
from Products.CMFCore.permissions import AddPortalContent
from Products.CMFCore.utils import ContentInit
from Products.Archetypes.atapi import listTypes
from Products.Archetypes.atapi import process_types

# Local imports
from Products.DataGridField.DataGridWidget  import DataGridWidget
from Products.DataGridField.DataGridField  import DataGridField
from Products.DataGridField.DataGridField  import FixedRow
from Products.DataGridField.Column import Column
from Products.DataGridField.SelectColumn import SelectColumn
from Products.DataGridField.RadioColumn import RadioColumn
from Products.DataGridField.FixedColumn import FixedColumn
from Products.DataGridField.LinkColumn import LinkColumn
from Products.DataGridField.HelpColumn import HelpColumn
from Products.DataGridField.CheckboxColumn import CheckboxColumn
from Products.DataGridField.LinesColumn import LinesColumn
from Products.DataGridField.DateColumn import DateColumn
from Products.DataGridField.DatetimeColumn import DatetimeColumn
from Products.DataGridField.DatetimeLocalColumn import DatetimeLocalColumn
from Products.DataGridField.FileColumn import FileColumn
from Products.DataGridField.EmailColumn import EmailColumn
from Products.DataGridField.ColorColumn import ColorColumn
from Products.DataGridField.PasswordColumn import PasswordColumn
from Products.DataGridField.RangeColumn import RangeColumn
from Products.DataGridField.MonthColumn import MonthColumn
from Products.DataGridField.SearchColumn import SearchColumn
from Products.DataGridField.TimeColumn import TimeColumn
from Products.DataGridField.UrlColumn import UrlColumn
from Products.DataGridField.WeekColumn import WeekColumn

from Products.DataGridField.config import PKG_NAME, GLOBALS

# Import "MessageFactory" to create messages in the DataGridField domain
DGFMessageFactory = MessageFactory('datagridfield')
# Import "MessageFactory" to create messages in the plone domain
PloneMessageFactory = MessageFactory('plone')

from Products.DataGridField import validators

registerDirectory('skins', GLOBALS)


def initialize(context):
    # Example content type initialization
    import Products.DataGridField.examples
    content_types, constructors, ftis = process_types(
        listTypes(PKG_NAME), PKG_NAME, )

    ContentInit(
        '%s Content' % PKG_NAME,
        content_types=content_types,
        permission=AddPortalContent,
        extra_constructors=constructors,
        fti=ftis,
        ).initialize(context)



"""

    Month column for DataGridField.

"""

__author__ = "T. Kim Nguyen <nguyen@uwosh.edu>"
__docformat__ = 'plaintext'

# Zope imports
from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from Products.DataGridField.Column import Column


class MonthColumn(Column):
    """ Month column """

    security = ClassSecurityInfo()

    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_month_cell"


# Initializes class security
InitializeClass(MonthColumn)



"""

    Copyright 2006 Red Innovation

"""

__author__ = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'restructuredtext'

from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from Products.DataGridField.Column import Column
from Products.DataGridField.utils import makeAbsoluteLink, makeRelativeLink


class LinkColumn(Column):
    """ Defines DataGridField column with link descriptions and targets

    Description and link is stored as one text entry, in of
    form which is used by many Wikis.

    Format:
        "description" "|" "link"

    If links begin with http:// they are absolute. Otherwise
    links are relative to the portal root.
    Managing absolute and relative links are done through utils module
    functions.
    """

    security = ClassSecurityInfo()

    def __init__(self, label, **kwargs):
        """ Create a Links

        @param linkClass CSS class for <a> links in view mode

        """
        linkClass = kwargs.pop('linkClass', '')
        Column.__init__(self, label, **kwargs)
        self.linkClass = linkClass

    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_link_cell"

    security.declarePublic('getLinkClass')
    def getLinkClass(self):
        return self.linkClass

    security.declarePublic('getLink')
    def getLink(self, instance, cell):
        """ Extract link address from cell data """

        if cell == None:
            return ""

        splitted = cell.split("|")
        if len(splitted) >= 2:
            return makeAbsoluteLink(splitted[1], instance)
        return ""

    security.declarePublic('getRelativeLink')
    def getRelativeLink(self, instance, cell):
        """ Extract link address from cell data """

        if cell == None:
            return ""

        splitted = cell.split("|")
        if len(splitted) >= 2:
            return makeRelativeLink(splitted[1], instance)
        return ""

    security.declarePublic("getDescription")
    def getDescription(self, cell):
        """ Extract link description from cell data """

        if cell == None:
            return ""

        splitted = cell.split("|")
        if len(splitted) >= 1:
            return splitted[0]
        return ""


    security.declarePublic('processCellData')
    def processCellData(self, form, value, context, field, columnId):
        """ Read cell values from raw form data

        Read form fields xxx_link and xx_desc and form one
        description from them.
        """

        # scan all rows and build desc|link string pair for
        # fields
        newValue = []
        for row in value:

            # we must clone row since
            # row is readonly ZPublished.HTTPRequest.record object
            newRow = {}
            for key in row.keys():
                newRow[key] = row[key]

            if newRow.has_key(columnId + "_desc"):
                desc = newRow[columnId + "_desc"]
            else:
                desc = ""

            if newRow.has_key(columnId + "_link"):
                link = newRow[columnId + "_link"]
            else:
                link = ""

            link = makeRelativeLink(link, context)

            if desc or link:
                newRow[columnId] = "%s|%s" % (desc, link)
            else:
                # don't add | character alone
                # causes extra new row to appear
                newRow[columnId] = ""

            newValue.append(newRow)

        return newValue

# Initializes class security
InitializeClass(LinkColumn)



"""Validator for DataGridFields.

For the DataGridFields, making them required is not enough as there is
always a hidden entry.  So we check if there is least one normal entry
and one hidden entry, so more than 1 entry in total.
"""

from Products.validation import validation
from Products.validation.interfaces.IValidator import IValidator
from Products.validation.i18n import recursiveTranslate
from zope.interface import implements

from Products.DataGridField import DGFMessageFactory as _
from Products.DataGridField.DataGridWidget import DataGridWidget

class DataGridValidator:
    """Validate as True when having at least one DataGrid item.
    """

    implements(IValidator)

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):
        try:
            length = len(value) - 1
        except TypeError:
            return ("Validation failed(%s): cannot calculate length "
                    "of %s.""" % (self.name, value))
        except AttributeError:
            return ("Validation failed(%s): cannot calculate length "
                    "of %s.""" % (self.name, value))
        if length < 1:
            return ("Validation failed(%s): Need at least one entry."
                    % self.name)
        return True

class ColumnRequiredDataGridValidator:
    """Validate as True when data inside a column is required
    """

    implements(IValidator)

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):
        field = kwargs['field']
        if not isinstance(field.widget, DataGridWidget):
            # can't apply
            return True
        required_cols = []
        for cid, cdef in field.widget.columns.items():
            if cdef.required:
                required_cols.append(cid)
        if not required_cols:
            return True
        missing_columns = []
        for row in value:
            if not row.get('orderindex_', '').isdigit():
                # skipping the "template_row_marker" when on edit
                continue
            for cname, cvalue in row.items():
                if cname in required_cols and not cvalue and cname not in missing_columns:
                    missing_columns.append(cname)
        if missing_columns:
            missing_column_labels = []
            for cname in missing_columns:
                # Try to get a translated label.
                try:
                    cdef = field.widget.columns.get(cname)
                    label = cdef.getLabel(context=kwargs.get('REQUEST'))
                except:
                    # fall back to the column id.
                    label = cname
                missing_column_labels.append(label)
            return recursiveTranslate(_('missing_columns',
                                        default = u"The following columns are required but not all rows "
                                                  u"have been filled: ${columns}",
                                        mapping = {'columns': ', '.join(missing_column_labels)})
                                      , **kwargs)
        return True


isDataGridFilled = DataGridValidator(
    'isDataGridFilled', title='DataGrid has entries',
    description='The DataGridField must have at least one entry.')
validation.register(isDataGridFilled)

isColumnFilled = ColumnRequiredDataGridValidator(
    'isColumnFilled', title='Required columns have entries',
    description='All required columns must have data in every rows.')
validation.register(isColumnFilled)



"""

    Basic columns for DataGridField.

    Copyright 2006-2007 DataGridField authors


"""

__author__ = "Mikko Ohtamaa < mikko@redinnovation.com>"
__docformat__ = 'epytext'

# Zope imports
from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from zope.i18n import translate


class Column(object):
    """ Stardard text edit column """

    security = ClassSecurityInfo()

    def __init__(self, label, col_description=None, default=None,
                 default_method=None, visible=True, required=False):
        """ Create a column

            @param label User visible name
            @param col_description General description for the column (when label is  not enough)
            @param default Default value for new rows
            @param default_method Default function to generate the default value for new rows
            @param visible Hide column from displaying by setting this to False
            @param required Set to True when values in this column are required
        """
        self.label = label
        self.col_description = col_description
        self.default = default
        self.default_method = default_method
        self.visible = visible
        self.required = required

    security.declarePublic('getLabel')
    def getLabel(self, context, widget=None):
        """ User friendly name for the column.

        This includes translation.

        @param context Context where translation happens (should be a request)

        @param widget The parent widget of this column. This is
        ignored now, as we do not support specifying an i18n_domain in
        the widget.  The labels should be specified with a message factory.
        """
        try:
            return translate(self.label, context=context)
        except:
            # Fall back to the untranslated label instead of crashing
            # simply because translation fails.
            return self.label

    security.declarePublic('getDefault')
    def getDefault(self, context):
        """ Default value for new rows """
        if self.default_method:
            try:
                func = getattr(context, self.default_method)
            except AttributeError:
                raise AttributeError, "Class %s has no default_method %s" % (str(context), self.default_method)

            return func()


        return self.default

    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_text_cell"

    security.declarePublic('processCellData')
    def processCellData(self, form, value, context, field, columnId):
        """ Read cell values from raw form data

        Column processing in forms may need special preparations for data if
        widgets use other than <input value> for storing their
        values in fields.

        @param form Submitted form, contains HTML fields
        @param context Archetypes item instance for the submitted form
        @param field Assigned field for this widget
        @param columnId Column what we are operating

        @return new values which are constructed by processing data
        """
        return value

# Initializes class security
InitializeClass(Column)



"""

    Column with help for DataGridField

    Written by Juan Grigera <juan@grigera.com.ar>.

"""

__docformat__ = 'epytext'
__author__ = "Juan Grigera <juan@grigera.com.ar>"

from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from Products.CMFCore.permissions import View
from Products.DataGridField.Column import Column


class HelpColumn(Column):
    """ Help column support.

    Behaves like normal text cell, but has a help pop-up icon next to it.
    """

    security = ClassSecurityInfo()

    def __init__(self, label, helper_text, script, icon, **kwargs):
        """ Create a HelpColumn

        """
        Column.__init__(self, label, **kwargs)
        self.helper_text = helper_text
        self.helper_url = script
        self.icon = icon

    security.declareProtected(View, 'getVocabulary')
    def getVocabulary(self, instance):
        """ Gets this column vocabulary for specific Archetypes instance
        """
        try:
            func = getattr(instance, self.vocabulary)
        except AttributeError:
            raise AttributeError, "Class %s is missing vocabulary function %s" % (str(instance), self.vocabulary)

        return func()


    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_help_cell"

    security.declarePublic('getHelperUrl')
    def getHelperUrl(self):
        """ Return url to open"""
        return self.helper_url

    security.declarePublic('getHelperText')
    def getHelperText(self):
        """ Return help text"""
        return self.helper_text

    security.declarePublic('getIcon')
    def getIcon(self):
        return self.icon

# Initializes class security
InitializeClass(HelpColumn)



"""

    Email column for DataGridField.

"""

__author__ = "T. Kim Nguyen <nguyen@uwosh.edu>"
__docformat__ = 'plaintext'

# Zope imports
from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from Products.DataGridField.Column import Column


class EmailColumn(Column):
    """ Email column """

    security = ClassSecurityInfo()

    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_email_cell"


# Initializes class security
InitializeClass(EmailColumn)



"""

    Small utility functions for DGF

    Copyright 2006 Red Innovation

"""

__author__ = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'restructuredtext'

from AccessControl import ModuleSecurityInfo, allow_module
from Products.CMFCore.utils import getToolByName


module = "Products.DataGridField.utils"
security = ModuleSecurityInfo(module)
allow_module(module)

security.declarePublic("makeAbsoluteLink")
def makeAbsoluteLink(link, context):
    """ Convert site relative links to absolute URLs

    Absolute links are passed through as is.
    Assume all http:// prefixed links are absolute.
    """

    if link == "":
        # this prevents makeAbsoluteLink not to
        # do nasty things with empty fields
        return ""

    # see if the link is absolute
    if link.startswith("http://"):
        return link
    else:
        portal_url = getToolByName(context, "portal_url")
        if link[:1] == "/":
            return portal_url() + link
        else:
            # if first character is / don't add it twice
            return portal_url() + "/" + link

security.declarePublic("makeRelativeLink")
def makeRelativeLink(link, context):
    """ Convert absolute (copy-pasted) links to site relative links

    If user enters absolute link which points to current portal,
    convert link to relative link so it survives portal relocation.

    Assume are http:// prefixed links are absolute.
    """

    # check for absolute, avoid extra work below
    if link.startswith("http://"):
        portal_url = getToolByName(context, "portal_url")()
        if link.startswith(portal_url):
            return link[len(portal_url):]

    # if first character is / don't add it twice
    return link



"""

    Lines column - used for list of values

    Licensed under GPL.

"""

from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from Products.DataGridField.Column import Column


class LinesColumn(Column):
    """ Textarea which returns list of lines

    Used eg. as vocabulary definition in PFGDataGrid
    """
    security = ClassSecurityInfo()

    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_lines_cell"

    security.declarePublic('processCellData')
    def processCellData(self, form, value, context, field, columnId):
        """ Read cell values from raw form data

        Column processing in forms may need special preparations for data if
        widgets use other than <input value> for storing their
        values in fields.

        @param form Submitted form, contains HTML fields
        @param context Archetypes item instance for the submitted form
        @param field Assigned field for this widget
        @param columnId Column what we are operating

        @return new values which are constructed by processing data
        """
        # scan all rows and build list of lines for fields
        newValue = []
        for row in value:

            # we must clone row since
            # row is readonly ZPublished.HTTPRequest.record object
            newRow = {}
            for key in row.keys():
                newRow[key] = row[key]

            # make list of values splited by \n with each item stripped
            newRow[columnId] = [
                v.strip() for v in newRow[columnId].split('\n')]
            # last item should not be empty
            if newRow[columnId] and newRow[columnId][-1] == '':
                newRow[columnId] = newRow[columnId][:-1]
            newValue.append(newRow)

        return newValue

# Initializes class security
InitializeClass(LinesColumn)



"""

    Radio button column

    Copyright 2006 Red Innovation

    Licensed under GPL.

"""

__author__ = "Mikko Ohtamaa <mikko@redinnovation.com>"
__docformat__ = 'reStructuredText'

from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from Products.DataGridField.SelectColumn import SelectColumn


class RadioColumn(SelectColumn):
    """ Allow user select one from many options using radio buttons.

    WARNING: Does not work with validation. All field values will be cleared
    when validation fails on the edit page submit. This is a limitation in
    the current architecture (Archetypes / ZPublisher / DataGridField).

    Some explanation hints:

    In::
        <input type="radio" name=""
    name must be unique among radio button group.

    This is problematic. ZPublisher cannot associate radio buttons to :records
    groups since :records groups are formed as::

        name string:${fieldName}.${column}:records;

    ZPublisher's :records naming convention doesn't allow different per row
    radio button groups. Instead, we assign an unique name for reach
    radio button cell using::

        name string:${fieldName}.${column}.${repeat/rows/number};

    Column.processCellData parses form data and propeply combines
    field value from radio buttons so that Archetypes' field framework
    correctly understand the set value.

    """
    security = ClassSecurityInfo()

    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_radio_cell"

    security.declarePublic('processCellData')
    def processCellData(self, form, value, context, field, columnId):
        """ Read cell values from raw form data

        Read special table for radio button columns from form data.
        The selected radio button cell id is placed as a cell value.
        """

        newValue = []

        #print "form value:" + str(form)

        for row in value:

            # we must clone row since
            # row is readonly ZPublished.HTTPRequest.record object
            newRow = {}
            for key in row.keys():
                newRow[key] = row[key]

            orderIndex = row["orderindex_"]
            cellId = "%s.%s.%s" % (field.getName(), columnId, orderIndex)
            if form.has_key(cellId):
                # If radio button is set in HTML form
                # it's id appears in form of field.column.orderIndex
                newRow[columnId] = form[cellId]

            newValue.append(newRow)

        return newValue




# Initializes class security
InitializeClass(RadioColumn)



from setuptools import setup, find_packages

version = '1.9.6'
readme = open("README.rst").read()
history = open("CHANGES.rst").read()
long_description = readme + "\n" + history


setup(name='Products.DataGridField',
      version=version,
      description="A table input component for Plone.",
      long_description=long_description,
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.6",
          "Programming Language :: Python :: 2.7",
          "Topic :: Software Development :: Libraries :: Python Modules",
          "Framework :: Plone",
          "Framework :: Plone :: 4.0",
          "Framework :: Plone :: 4.1",
          "Framework :: Plone :: 4.2",
          "Framework :: Plone :: 4.3",
          "Framework :: Zope2",
          "Development Status :: 5 - Production/Stable",
          ],
      keywords='Plone DataGridField Archetypes',
      author='Jarn',
      author_email='info@jarn.com',
      maintainer='Mikko Ohtamaa',
      maintainer_email='info@redinnovation.com',
      url='http://plone.org/products/datagridfield',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
      ],
      extras_require=dict(
          test=[
              # Yes, we need both CMFTestCase and PloneTestCase:
              'Products.CMFTestCase',
              'Products.PloneTestCase',
              ]
      ),
      )



"""

    Fixed text column

    Copyright 2006 Red Innovation

    Licensed under GPL.

"""

__author__ = "Mikko Ohtamaa <mikko@redinnovation.com>"
__docformat__ = 'reStructuredText'

from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from Products.DataGridField.Column import Column

class FixedColumn(Column):
    """ Column with non-changeable text

    Useful with DataGridField.fixed_row property in some use cases.
    """
    security = ClassSecurityInfo()

    security.declarePublic('getMacro')
    def getMacro(self):
        """ Return macro used to render this column in view/edit """
        return "datagrid_fixed_cell"


# Initializes class security
InitializeClass(FixedColumn)



from Products.DataGridField.tests.DataGridTestCase import DataGridTestCase


class TestSetup(DataGridTestCase):
    """ Unit test cases for DataGridField setup and installation """

    def testCSSNamespaceCollosion(self):
        """ See Issue #30 at
            http://plone.org/products/datagridfield/issues/30
        """
        dgf_skin_layer = getattr(self.portal.portal_skins, 'DataGridWidget')
        self.failUnless(dgf_skin_layer, "The DataGridWidget skin layer "
            " wasn't even installed within the site.")
        self.failIf(dgf_skin_layer.objectIds().count('datagridwidget.css') > 1)

    def testNamespaceCollidingCSSFilesMerged(self):
        """ And an additional follow-up to Issue #30. We have non-overlapping
            CSS declarations in a datagridwidget.css.dtml and
            datagridwidget.css file.
        """
        # datagridwidget.css.dtml handles many of the editing/view
        # interactions with DGF for example display: none of hidden
        # columns so we confirm:
        #   - td.datagridwidget-hidden-data
        raw_css = self.portal.restrictedTraverse('datagridwidget.css').raw
        self.failUnless('td.datagridwidget-hidden-data' in raw_css)

        # by contrast datagridwidget.css applies styles to what appear to be
        # autogenerated classes on the Archetypes base edit.  We maintain
        # these for BBB. Confirm:
        #   - div.ArchetypesDataGridWidget table td img
        self.failUnless('div.ArchetypesDataGridWidget table td img' in raw_css)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestSetup))
    return suite



"""

    Unit testing for DataGridField product

    Copyright 2006 DataGridField authors

"""

import sys
import logging

# Output debug log
logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
logging.getLogger("DataGridField").setLevel(logging.DEBUG)

from Products.DataGridField.tests.DataGridTestCase import DataGridTestCase
from Products.Archetypes.atapi import DisplayList

from Products.DataGridField.utils import makeAbsoluteLink, makeRelativeLink


class TestInstallation(DataGridTestCase):

    def afterSetUp(self):
        types = self.portal.portal_types.objectIds()
        self.failUnless('DataGridDemoType' in types)

    def testPortalTypes(self):
        types = self.portal.portal_types.objectIds()
        self.failUnless('DataGridDemoType' in types)

    def testPortalSkins(self):
        skins = self.portal.portal_skins.objectIds()
        self.failUnless('DataGridWidget' in skins)

    def testAddingDemo(self):
        self.folder.invokeFactory('DataGridDemoType', 'foo')
        self.failUnless('foo' in self.folder.objectIds())

    def testSettingField(self):
        self.folder.invokeFactory('DataGridDemoType', 'foo')
        vals = [{'column1':'joe'}]
        self.folder.foo.setDemoField(vals)
        self.assertEqual(len(self.folder.foo.getDemoField()), 1)

        row1 = self.folder.foo.getDemoField()[0]
        self.failUnless('column1' in row1.keys(), "Row was " + str(row1))
        pair1 = row1['column1']
        self.assertEqual(pair1, vals[0]['column1'])

    def testSettingFieldWithOrder(self):
        self.folder.invokeFactory('DataGridDemoType', 'foo')
        vals = [{'column1': 'joe', 'orderindex_': 1},
                {'column1': 'jack', 'orderindex_': 0}]
        self.folder.foo.setDemoField(vals)
        self.assertEqual(len(self.folder.foo.getDemoField()), 2)
        self.assertEqual(self.folder.foo.getDemoField()[0]['column1'],
                         vals[1]['column1'])
        self.assertEqual(self.folder.foo.getDemoField()[1]['column1'],
                         vals[0]['column1'])

    def testSettingEmptyRows(self):
        """ It should be possible to set empty rows """
        self.folder.invokeFactory('DataGridDemoType', 'foo')

        data = {'The third': '', 'column1': 'xxx', 'column2': ''}
        raw_data = data.copy()
        raw_data['orderindex_'] = '1'
        self.folder.foo.setDemoField((raw_data,))
        self.assertEqual(self.folder.foo.getDemoField(), (data,))

        data = {'The third': '', 'column1': '', 'column2': ''}
        raw_data = data.copy()
        raw_data['orderindex_'] = '1'
        self.folder.foo.setDemoField((raw_data,))
        self.assertEqual(self.folder.foo.getDemoField(), (data,))

        obj = self.folder.foo
        field = obj.Schema()['DemoField']
        errors = {}
        self.assertEqual(field.validate_required(obj, (raw_data,), errors), None)
        self.assertEqual(errors, {})

        # validate_required stored data, it will miss the 'orderindex_' key
        del raw_data['orderindex_']
        self.assertEqual(field.validate_required(obj, (raw_data,), errors), None)
        self.assertEqual(errors, {})

    def testSettingEmptyTable(self):
        """ It should not be possible to set no rows at all when field is required """
        self.folder.invokeFactory('DataGridDemoType', 'foo')

        data = {'The third': '', 'column1': '', 'column2': ''}
        raw_data = data.copy()
        raw_data['orderindex_'] = 'template_row_marker'  # whatever is not a position digit
        self.folder.foo.setDemoField((raw_data,))
        self.assertEqual(self.folder.foo.getDemoField(), tuple())

        obj = self.folder.foo
        field = obj.Schema()['DemoField']
        errors = {}
        self.assertEqual(str(field.validate_required(obj, (raw_data,), errors)),
                         'DemoField is required, please correct.')
        self.assertTrue(errors)
        self.assertTrue('DemoField' in errors)
        self.assertEqual(errors['DemoField'],
                         'DemoField is required, please correct.')

    def testRenderingDoesNotFail(self):
        """See if a page containing DGF will output HTML without
        exceptions in view mode.
        """
        self.folder.invokeFactory('DataGridDemoType', 'foo')
        self.failUnless(self.folder.foo())


class TestGridAPI(DataGridTestCase):

    def afterSetUp(self):

        self.folder.invokeFactory('DataGridDemoType', 'demo')
        self.demo = self.folder.demo
        self.demo.setDemoField([
                {'column1': 'a', 'column2': 'b', 'The third': 'c'},
                {'column1': 'd', 'column2': 'e', 'The third': 'f'}])
        self.field = self.demo.getField('DemoField')

    def testGetColumns(self):
        columns = self.field.getColumns(self.demo)
        self.assertEqual(columns[0], 'column1')
        self.assertEqual(columns[1], 'column2')
        self.assertEqual(columns[2], 'The third')

    def testGetRowCount(self):
        self.assertEqual(self.field.getRowCount(self.demo), 2)

    def testGetRow(self):
        self.assertEqual(
            self.field.getRow(self.demo, 1),
            {'column1': 'd', 'column2': 'e', 'The third': 'f'})

    def testGetColumn(self):
        self.assertEqual(
            tuple(self.field.getColumn(self.demo, 'column2')),
            ('b', 'e',))

    def testSearchKey(self):
        self.assertEqual(
            self.field.search(self.demo, key='d'),
            ({'column1': 'd', 'column2': 'e', 'The third': 'f'}, ))

    def testSearchColumn(self):
        self.assertEqual(
            self.field.search(self.demo, column2='e'),
            ({'column1': 'd', 'column2': 'e', 'The third': 'f'}, ))

    def testSearchWithNoResults(self):
        self.assertEqual(self.field.search(self.demo, 'foo'), ())

    def testLookup(self):
        self.assertEqual(self.field.lookup(self.demo, 'd', 'column2'), 'e')

    def testLookupWithKeyColumn(self):
        self.assertEqual(
            self.field.lookup(self.demo, 'b', 'column1', 'column2'), 'a')

    def testLookupNotFound(self):
        self.assertEqual(self.field.lookup(self.demo, 'x', 'The third'), None)

    def testGetAsDisplayList(self):
        lst = DisplayList()
        lst.add('a', 'b')
        lst.add('d', 'e')
        self.assertEqual(self.field.getAsDisplayList(self.demo), lst)

    def testGetAsDisplayListWithCustomColumns(self):
        lst = DisplayList()
        lst.add('c', 'a')
        lst.add('f', 'd')
        self.assertEqual(
            self.field.getAsDisplayList(self.demo, 'The third', 'column1'),
            lst)

    def testGetAsGrid(self):
        self.assertEqual(tuple(self.field.getAsGrid(self.demo)),
                         (('a', 'b', 'c',),
                          ('d', 'e', 'f')))

    def testQueryDataGrid(self):
        self.assertEqual(
            self.demo.queryDataGrid('DemoField', 'lookup', ['d', 'column2']),
            'e')

    def testSearch(self):
        catalog_tool = self.portal.portal_catalog
        self.folder.invokeFactory('DataGridDemoType2', 'searchme')
        self.folder.searchme.setDemoField2([
                {'column1': 'randomSearchableWord', 'column2': 'b',
                 'select_sample': 'sample'},
                {'column1': 'd', 'column2': 'e', 'select_sample': 'sample'}])
        self.folder.searchme.reindexObject()
        self.failUnless(
            'randomSearchableWord' in self.folder.searchme.SearchableText())
        brains = catalog_tool(SearchableText='randomSearchableWord')
        self.failUnless(len(brains) > 0)


class TestSpecialRowsBehavior(DataGridTestCase):
    """ Test fixed rows property, deletion locking, etc. """

    def afterSetUp(self):
        self.folder.invokeFactory('FixedRowsDemoType', 'demo')
        self.demo = self.folder.demo

        self.field = self.demo.getField('DemoField')
        self.failUnless(self.field != None)

        self.restrictedField = self.demo.getField('predefinedSkills')
        self.failUnless(self.restrictedField != None,
                        "No restricted field in demo type")

    def checkForCellValue(self, data, columnId, value):
        for row in data:
            if columnId in row.keys():
                if row[columnId] == value:
                    return True
        self.fail("Missing value %s for column %s" % (value, columnId))

    def testFixedRows(self):
        """Test that fixed rows are there always and non-key values
        can be changed.
        """

        # initial data get should be empty + fixed rows present
        data = self.field.get(self.demo)

        self.checkForCellValue(data, "column1", "must-exist-1")
        self.checkForCellValue(data, "column2", "must-exist-2")

    def testAddNonFixedRows(self):
        """ set more row and see that fixed rows are still present """
        self.demo.setDemoField([
                {'column1': 'a', 'column2': 'b', 'The third': 'c'},
                {'column1': 'd', 'column2': 'e', 'The third': 'f'}])

        data = self.field.get(self.demo)
        self.checkForCellValue(data, "column1", "must-exist-1")
        self.checkForCellValue(data, "column2", "must-exist-2")
        self.checkForCellValue(data, "column1", "a")
        self.checkForCellValue(data, "column2", "e")

    def testSetFixedRowValue(self):
        """ set non-key value of fixed row"""
        self.demo.setDemoField([
                {'column1': 'must-exist-1', 'column2': 'testvalue'}])
        data = self.field.get(self.demo)
        self.checkForCellValue(data, "column1", "must-exist-1")
        self.checkForCellValue(data, "column2", "must-exist-2")
        self.checkForCellValue(data, "column2", "testvalue")

        # test setting data outside fixed row definition
        self.demo.setDemoField([{'column1': 'naah', 'column2': 'nooh'}])
        data = self.field.get(self.demo)
        self.checkForCellValue(data, "column1", "naah")
        self.checkForCellValue(data, "column2", "nooh")

    def testDeleteRestrictedField(self):
        self.assertEqual(self.restrictedField.widget.isDeleteEnabled(
                self.demo, self.restrictedField), False)

    def testInsertRestrictedField(self):
        self.assertEqual(self.restrictedField.widget.isInsertEnabled(
                self.demo, self.restrictedField), False)

    def testReorderRestrictedField(self):
        self.assertEqual(self.restrictedField.widget.isReorderEnabled(
                self.demo, self.restrictedField), False)

    def testSetStringValue(self):
        self.demo.setDemoField('({"column1":"must-exist-1","column2":"testvalue"})')
        data = self.field.get(self.demo)
        self.checkForCellValue(data, "column1", "must-exist-1")
        self.checkForCellValue(data, "column2", "testvalue")

        self.demo.setDemoField("[{'column1': 'must-exist-1', 'column2': 'testvalue'}]")
        data = self.field.get(self.demo)
        self.checkForCellValue(data, "column1", "must-exist-1")
        self.checkForCellValue(data, "column2", "testvalue")

        self.demo.setDemoField('[{"column1": "must-exist-1", "column2": "testvalue"}]')
        data = self.field.get(self.demo)
        self.checkForCellValue(data, "column1", "must-exist-1")
        self.checkForCellValue(data, "column2", "testvalue")

        self.demo.setDemoField('({"column1": "must-exist-1", "column2": "testvalue"})')
        data = self.field.get(self.demo)
        self.checkForCellValue(data, "column1", "must-exist-1")
        self.checkForCellValue(data, "column2", "testvalue")

        # test with many lines

        self.demo.setDemoField('({"column1":"must-exist-1","column2":"testvalue"}, {"column1":"must-exist-2","column2":"testvalue2"})')
        value = self.demo.getDemoField()
        self.assertEqual(value[0]['column1'], "must-exist-1")
        self.assertEqual(value[0]['column2'], "testvalue")
        self.assertEqual(value[1]['column1'], "must-exist-2")
        self.assertEqual(value[1]['column2'], "testvalue2")

class TestLinkUtils(DataGridTestCase):
    """ Test LinkColumn link transforms """

    def afterSetUp(self):
        self.folder.invokeFactory("DataGridDemoType", id="sample")

    def testMakeAbsoluteURL(self):
        """ Test converting site relative links to full URLs
        """

        context = self.folder.sample

        self.assertEqual(makeAbsoluteLink("test", context),
                         "http://nohost/plone/test")
        self.assertEqual(makeAbsoluteLink("/test", context),
                         "http://nohost/plone/test")
        self.assertEqual(makeAbsoluteLink("http://test", context),
                         "http://test")

    def testMakeRelativeURL(self):
        """ Test converting full URLs to site relative links
        """

        context = self.folder.sample

        self.assertEqual(makeRelativeLink("http://nohost/plone/test", context),
                         "/test")
        self.assertEqual(makeRelativeLink("/test", context), "/test")
        self.assertEqual(makeRelativeLink("test", context), "test")
        self.assertEqual(makeRelativeLink("http://test", context),
                         "http://test")


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInstallation))
    suite.addTest(makeSuite(TestGridAPI))
    suite.addTest(makeSuite(TestSpecialRowsBehavior))
    suite.addTest(makeSuite(TestLinkUtils))
    return suite



from Testing import ZopeTestCase

ZopeTestCase.installProduct('DataGridField')

from Products.PloneTestCase import PloneTestCase

PloneTestCase.setupPloneSite(extension_profiles=[
        'Products.DataGridField:default',
        'Products.DataGridField:example'])


class DataGridTestCase(PloneTestCase.PloneTestCase):

    class Session(dict):

        def set(self, key, value):
            self[key] = value

    def _setup(self):
        PloneTestCase.PloneTestCase._setup(self)
        self.app.REQUEST['SESSION'] = self.Session()
